@@ -40,6 +40,7 @@
 
 #include "application.h"
 #include "consoleprogressobserver.h"
+#include "session.h"
 #include "status.h"
 #include "parser/commandlineoption.h"
 #include "../shared/logging/consolelogger.h"
@@ -77,6 +78,7 @@ CommandLineFrontend::CommandLineFrontend(const CommandLineParser &parser, Settin
 CommandLineFrontend::~CommandLineFrontend()
 {
     m_cancelTimer->stop();
+    delete m_observer;
 }
 
 // Called from interrupt handler. Don't do anything non-trivial here.
@@ -129,6 +131,10 @@ void CommandLineFrontend::start()
                 throw ErrorInfo(error);
             }
             break;
+        case SessionCommandType: {
+            startSession();
+            return;
+        }
         default:
             break;
         }
@@ -230,7 +236,7 @@ void CommandLineFrontend::handleJobFinished(bool success, AbstractJob *job)
                 return;
             }
             cancel();
-        } else if (SetupProjectJob * const setupJob = qobject_cast<SetupProjectJob *>(job)) {
+        } else if (const auto setupJob = qobject_cast<SetupProjectJob * const>(job)) {
             m_resolveJobs.removeOne(job);
             m_projects.push_back(setupJob->project());
             if (m_observer && resolvingMultipleProjects())
@@ -345,8 +351,7 @@ CommandLineFrontend::ProductMap CommandLineFrontend::productsToUse() const
     for (const Project &project : qAsConst(m_projects)) {
         QList<ProductData> &productList = products[project];
         const ProjectData projectData = project.projectData();
-        const auto products = projectData.allProducts();
-        for (const ProductData &product : products) {
+        for (const ProductData &product : projectData.allProducts()) {
             productNames << product.name();
             if (useAll || m_parser.products().contains(product.name())) {
                 productList.push_back(product);
@@ -418,6 +423,7 @@ void CommandLineFrontend::handleProjectsResolved()
         break;
     case HelpCommandType:
     case VersionCommandType:
+    case SessionCommandType:
         Q_ASSERT_X(false, Q_FUNC_INFO, "Impossible.");
     }
 }
@@ -611,7 +617,7 @@ void CommandLineFrontend::connectBuildJob(AbstractJob *job)
 {
     connectJob(job);
 
-    BuildJob *bjob = qobject_cast<BuildJob *>(job);
+    const auto bjob = qobject_cast<const BuildJob *>(job);
     if (!bjob)
         return;
 
@@ -640,18 +646,16 @@ ProductData CommandLineFrontend::getTheOneRunnableProduct()
     QBS_CHECK(m_projects.size() == 1); // Has been checked earlier.
 
     if (m_parser.products().size() == 1) {
-        const auto products = m_projects.front().projectData().allProducts();
-        for (const ProductData &p : products) {
+        for (const ProductData &p : m_projects.front().projectData().allProducts()) {
             if (p.name() == m_parser.products().constFirst())
                 return p;
         }
         QBS_CHECK(false);
     }
-    QBS_CHECK(m_parser.products().size() == 0);
+    QBS_CHECK(m_parser.products().isEmpty());
 
     QList<ProductData> runnableProducts;
-    const auto products = m_projects.front().projectData().allProducts();
-    for (const ProductData &p : products) {
+    for (const ProductData &p : m_projects.front().projectData().allProducts()) {
         if (p.isRunnable())
             runnableProducts.push_back(p);
     }
