@@ -1,6 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2019 Jochen Ulrich <jochenulrich@t-online.de>
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
@@ -46,13 +47,16 @@
 
 #include <language/language.h>
 #include <language/preparescriptobserver.h>
+#include <language/resolvedfilecontext.h>
 #include <language/scriptengine.h>
 #include <logging/logger.h>
 #include <tools/codelocation.h>
 #include <tools/error.h>
 #include <tools/qbsassert.h>
+#include <tools/qttools.h>
 
 #include <QtCore/qeventloop.h>
+#include <QtCore/qpointer.h>
 #include <QtCore/qthread.h>
 #include <QtCore/qtimer.h>
 
@@ -61,7 +65,7 @@ namespace Internal {
 
 struct JavaScriptCommandResult
 {
-    bool success;
+    bool success = false;
     QString errorMessage;
     CodeLocation errorLocation;
 };
@@ -70,9 +74,9 @@ class JsCommandExecutorThreadObject : public QObject
 {
     Q_OBJECT
 public:
-    JsCommandExecutorThreadObject(const Logger &logger)
-        : m_logger(logger)
-        , m_scriptEngine(0)
+    JsCommandExecutorThreadObject(Logger logger)
+        : m_logger(std::move(logger))
+        , m_scriptEngine(nullptr)
     {
     }
 
@@ -81,10 +85,13 @@ public:
         return m_result;
     }
 
-    void cancel()
+    void cancel(const qbs::ErrorInfo &reason)
     {
-        QBS_ASSERT(m_scriptEngine, return);
-        m_scriptEngine->abortEvaluation();
+        m_result.success = !reason.hasError();
+        m_result.errorMessage = reason.toString();
+        if (m_scriptEngine)
+            m_scriptEngine->abortEvaluation();
+        m_cancelled = true;
     }
 
 signals:
@@ -93,12 +100,19 @@ signals:
 public:
     void start(const JavaScriptCommand *cmd, Transformer *transformer)
     {
+        if (m_cancelled) {
+            emit finished();
+            return;
+        }
+
+        m_running = true;
         try {
             doStart(cmd, transformer);
         } catch (const qbs::ErrorInfo &error) {
             setError(error.toString(), cmd->codeLocation());
         }
 
+        m_running = false;
         emit finished();
     }
 
@@ -110,17 +124,20 @@ private:
         ScriptEngine * const scriptEngine = provideScriptEngine();
         QScriptValue scope = scriptEngine->newObject();
         scope.setPrototype(scriptEngine->globalObject());
-        PrepareScriptObserver observer(scriptEngine);
-        setupScriptEngineForFile(scriptEngine, transformer->rule->prepareScript->fileContext, scope);
+        m_scriptEngine->clearRequestedProperties();
+        setupScriptEngineForFile(scriptEngine,
+                                 transformer->rule->prepareScript.fileContext(), scope,
+                                 ObserveMode::Enabled);
 
         QScriptValue importScopeForSourceCode;
         if (!cmd->scopeName().isEmpty())
             importScopeForSourceCode = scope.property(cmd->scopeName());
 
-        setupScriptEngineForProduct(scriptEngine, transformer->product(), transformer->rule->module, scope,
-                                    &observer);
+        setupScriptEngineForProduct(scriptEngine, transformer->product().get(),
+                                    transformer->rule->module.get(), scope, true);
         transformer->setupInputs(scope);
-        transformer->setupOutputs(scriptEngine, scope);
+        transformer->setupOutputs(scope);
+        transformer->setupExplicitlyDependsOn(scope);
 
         for (QVariantMap::const_iterator it = cmd->properties().constBegin();
                 it != cmd->properties().constEnd(); ++it) {
@@ -131,13 +148,25 @@ private:
         if (importScopeForSourceCode.isObject())
             scriptEngine->currentContext()->pushScope(importScopeForSourceCode);
         scriptEngine->evaluate(cmd->sourceCode());
+        scriptEngine->releaseResourcesOfScriptObjects();
         if (importScopeForSourceCode.isObject())
             scriptEngine->currentContext()->popScope();
         scriptEngine->setGlobalObject(scope.prototype());
         transformer->propertiesRequestedInCommands
                 += scriptEngine->propertiesRequestedInScript();
-        transformer->propertiesRequestedFromArtifactInCommands
-                = scriptEngine->propertiesRequestedFromArtifact();
+        unite(transformer->propertiesRequestedFromArtifactInCommands,
+              scriptEngine->propertiesRequestedFromArtifact());
+        const std::vector<QString> &importFilesUsedInCommand
+                = scriptEngine->importedFilesUsedInScript();
+        transformer->importedFilesUsedInCommands.insert(
+                    transformer->importedFilesUsedInCommands.cend(),
+                    importFilesUsedInCommand.cbegin(), importFilesUsedInCommand.cend());
+        transformer->depsRequestedInCommands.add(scriptEngine->productsWithRequestedDependencies());
+        transformer->artifactsMapRequestedInCommands.unite(scriptEngine->requestedArtifacts());
+        for (const ResolvedProduct * const p : scriptEngine->requestedExports()) {
+            transformer->exportedModulesAccessedInCommands.insert(
+                        std::make_pair(p->uniqueName(), p->exportedModule));
+        }
         scriptEngine->clearRequestedProperties();
         if (scriptEngine->hasUncaughtException()) {
             // ### We don't know the line number of the command's sourceCode property assignment.
@@ -155,13 +184,15 @@ private:
     ScriptEngine *provideScriptEngine()
     {
         if (!m_scriptEngine)
-            m_scriptEngine = new ScriptEngine(m_logger, EvalContext::JsCommand, this);
+            m_scriptEngine = ScriptEngine::create(m_logger, EvalContext::JsCommand, this);
         return m_scriptEngine;
     }
 
     Logger m_logger;
     ScriptEngine *m_scriptEngine;
     JavaScriptCommandResult m_result;
+    bool m_running = false;
+    bool m_cancelled = false;
 };
 
 
@@ -171,6 +202,9 @@ JsCommandExecutor::JsCommandExecutor(const Logger &logger, QObject *parent)
     , m_objectInThread(new JsCommandExecutorThreadObject(logger))
     , m_running(false)
 {
+    qRegisterMetaType<Transformer *>("Transformer *");
+    qRegisterMetaType<const JavaScriptCommand *>("const JavaScriptCommand *");
+
     m_objectInThread->moveToThread(m_thread);
     connect(m_objectInThread, &JsCommandExecutorThreadObject::finished,
             this, &JsCommandExecutor::onJavaScriptCommandFinished);
@@ -181,12 +215,12 @@ JsCommandExecutor::JsCommandExecutor(const Logger &logger, QObject *parent)
 JsCommandExecutor::~JsCommandExecutor()
 {
     waitForFinished();
-    delete m_objectInThread;
     m_thread->quit();
     m_thread->wait();
+    delete m_objectInThread;
 }
 
-void JsCommandExecutor::doReportCommandDescription()
+void JsCommandExecutor::doReportCommandDescription(const QString &productName)
 {
     if ((m_echoMode == CommandEchoModeCommandLine
          || m_echoMode == CommandEchoModeCommandLineWithEnvironment)
@@ -195,7 +229,7 @@ void JsCommandExecutor::doReportCommandDescription()
         return;
     }
 
-    AbstractCommandExecutor::doReportCommandDescription();
+    AbstractCommandExecutor::doReportCommandDescription(productName);
 }
 
 void JsCommandExecutor::waitForFinished()
@@ -203,28 +237,32 @@ void JsCommandExecutor::waitForFinished()
     if (!m_running)
         return;
     QEventLoop loop;
-    connect(m_objectInThread, &JsCommandExecutorThreadObject::finished, &loop, &QEventLoop::quit);
+    connect(this, &AbstractCommandExecutor::finished, &loop, &QEventLoop::quit);
     loop.exec();
 }
 
-void JsCommandExecutor::doStart()
+bool JsCommandExecutor::doStart()
 {
-    QBS_ASSERT(!m_running, return);
-    m_thread->start();
+    QBS_ASSERT(!m_running, return false);
 
     if (dryRun() && !command()->ignoreDryRun()) {
         QTimer::singleShot(0, this, [this] { emit finished(); }); // Don't call back on the caller.
-        return;
+        return false;
     }
 
+    m_thread->start();
     m_running = true;
     emit startRequested(jsCommand(), transformer());
+    return true;
 }
 
-void JsCommandExecutor::cancel()
+void JsCommandExecutor::cancel(const qbs::ErrorInfo &reason)
 {
-    if (!dryRun())
-        QTimer::singleShot(0, m_objectInThread, [this] { m_objectInThread->cancel(); });
+    if (m_running && (!dryRun() || command()->ignoreDryRun()))
+        QTimer::singleShot(0, m_objectInThread, [objectInThread = QPointer<JsCommandExecutorThreadObject>{m_objectInThread}, reason] {
+            if (objectInThread)
+                objectInThread->cancel(reason);
+        });
 }
 
 void JsCommandExecutor::onJavaScriptCommandFinished()
