@@ -125,7 +125,7 @@ ItemDeclaration BuiltinDeclarations::declarationsForType(ItemType type) const
 }
 
 ItemType BuiltinDeclarations::typeForName(const QString &typeName,
-                                          const CodeLocation location) const
+                                          const CodeLocation &location) const
 {
     const auto it = m_typeMap.constFind(typeName);
     if (it == m_typeMap.constEnd())
@@ -150,7 +150,8 @@ QStringList BuiltinDeclarations::argumentNamesForScriptFunction(ItemType itemTyp
                                                                 const QString &scriptName) const
 {
     const ItemDeclaration itemDecl = declarationsForType(itemType);
-    for (const PropertyDeclaration &propDecl : itemDecl.properties()) {
+    const auto properties = itemDecl.properties();
+    for (const PropertyDeclaration &propDecl : properties) {
         if (propDecl.name() == scriptName)
             return propDecl.functionArgumentNames();
     }
