@@ -41,95 +41,112 @@
 
 #include <memory>
 
+namespace qbs {
+namespace Internal {
+
+enum class ObserveMode;
+
+class Value;
+using ValuePtr = std::shared_ptr<Value>;
+using ValueConstPtr = std::shared_ptr<const Value>;
+
+class ItemValue;
+using ItemValuePtr = std::shared_ptr<ItemValue>;
+using ItemValueConstPtr = std::shared_ptr<const ItemValue>;
+
+class JSSourceValue;
+using JSSourceValuePtr = std::shared_ptr<JSSourceValue>;
+using JSSourceValueConstPtr = std::shared_ptr<const JSSourceValue>;
+
+class VariantValue;
+using VariantValuePtr = std::shared_ptr<VariantValue>;
+using VariantValueConstPtr = std::shared_ptr<const VariantValue>;
+
+class FileContext;
+using FileContextPtr = std::shared_ptr<FileContext>;
+using FileContextConstPtr = std::shared_ptr<const FileContext>;
+
+class FileContextBase;
+using FileContextBasePtr = std::shared_ptr<FileContextBase>;
+using FileContextBaseConstPtr = std::shared_ptr<const FileContextBase>;
+
+class Probe;
+using ProbePtr = std::shared_ptr<Probe>;
+using ProbeConstPtr = std::shared_ptr<const Probe>;
+
+class PropertyMapInternal;
+using PropertyMapPtr = std::shared_ptr<PropertyMapInternal>;
+using PropertyMapConstPtr = std::shared_ptr<const PropertyMapInternal>;
+
+class FileTagger;
+using FileTaggerPtr = std::shared_ptr<FileTagger>;
+using FileTaggerConstPtr = std::shared_ptr<const FileTagger>;
+
+class ResolvedProduct;
+using ResolvedProductPtr = std::shared_ptr<ResolvedProduct>;
+using ResolvedProductConstPtr = std::shared_ptr<const ResolvedProduct>;
+
+class ResolvedProject;
+using ResolvedProjectPtr = std::shared_ptr<ResolvedProject>;
+using ResolvedProjectConstPtr = std::shared_ptr<const ResolvedProject>;
+
+class TopLevelProject;
+using TopLevelProjectPtr = std::shared_ptr<TopLevelProject>;
+using TopLevelProjectConstPtr = std::shared_ptr<const TopLevelProject>;
+
+class ResolvedFileContext;
+using ResolvedFileContextPtr = std::shared_ptr<ResolvedFileContext>;
+using ResolvedFileContextConstPtr = std::shared_ptr<const ResolvedFileContext>;
+
+class Rule;
+using RulePtr = std::shared_ptr<Rule>;
+using RuleConstPtr = std::shared_ptr<const Rule>;
+
+class ResolvedScanner;
+using ResolvedScannerPtr = std::shared_ptr<ResolvedScanner>;
+using ResolvedScannerConstPtr = std::shared_ptr<const ResolvedScanner>;
+
+class SourceArtifactInternal;
+using SourceArtifactPtr = std::shared_ptr<SourceArtifactInternal>;
+using SourceArtifactConstPtr = std::shared_ptr<const SourceArtifactInternal>;
+
+class ScriptFunction;
+using ScriptFunctionPtr = std::shared_ptr<ScriptFunction>;
+using ScriptFunctionConstPtr = std::shared_ptr<const ScriptFunction>;
+class PrivateScriptFunction;
+
+class RuleArtifact;
+using RuleArtifactPtr = std::shared_ptr<RuleArtifact>;
+using RuleArtifactConstPtr = std::shared_ptr<const RuleArtifact>;
+
+class ResolvedModule;
+using ResolvedModulePtr = std::shared_ptr<ResolvedModule>;
+using ResolvedModuleConstPtr = std::shared_ptr<const ResolvedModule>;
+
+class ResolvedGroup;
+using GroupPtr = std::shared_ptr<ResolvedGroup>;
+using GroupConstPtr = std::shared_ptr<const ResolvedGroup>;
+
+class ArtifactProperties;
+using ArtifactPropertiesPtr = std::shared_ptr<ArtifactProperties>;
+using ArtifactPropertiesConstPtr = std::shared_ptr<const ArtifactProperties>;
+
+class ExportedItem;
+using ExportedItemPtr = std::shared_ptr<ExportedItem>;
+class ExportedModule;
+class ExportedModuleDependency;
+class ExportedProperty;
+class PersistentPool;
+
+} // namespace Internal
+} // namespace qbs
+
+#ifdef QT_CORE_LIB
 #include <QtCore/qhash.h>
 
 namespace qbs {
 namespace Internal {
 
-class Value;
-typedef std::shared_ptr<Value> ValuePtr;
-typedef std::shared_ptr<const Value> ValueConstPtr;
-
-class ItemValue;
-typedef std::shared_ptr<ItemValue> ItemValuePtr;
-typedef std::shared_ptr<const ItemValue> ItemValueConstPtr;
-
-class JSSourceValue;
-typedef std::shared_ptr<JSSourceValue> JSSourceValuePtr;
-typedef std::shared_ptr<const JSSourceValue> JSSourceValueConstPtr;
-
-class VariantValue;
-typedef std::shared_ptr<VariantValue> VariantValuePtr;
-typedef std::shared_ptr<const VariantValue> VariantValueConstPtr;
-
-class FileContext;
-typedef std::shared_ptr<FileContext> FileContextPtr;
-typedef std::shared_ptr<const FileContext> FileContextConstPtr;
-
-class FileContextBase;
-typedef std::shared_ptr<FileContextBase> FileContextBasePtr;
-typedef std::shared_ptr<const FileContextBase> FileContextBaseConstPtr;
-
-class Probe;
-typedef std::shared_ptr<Probe> ProbePtr;
-typedef std::shared_ptr<const Probe> ProbeConstPtr;
-
-class PropertyMapInternal;
-typedef std::shared_ptr<PropertyMapInternal> PropertyMapPtr;
-typedef std::shared_ptr<const PropertyMapInternal> PropertyMapConstPtr;
-
-class FileTagger;
-typedef std::shared_ptr<FileTagger> FileTaggerPtr;
-typedef std::shared_ptr<const FileTagger> FileTaggerConstPtr;
-
-class ResolvedProduct;
-typedef std::shared_ptr<ResolvedProduct> ResolvedProductPtr;
-typedef std::shared_ptr<const ResolvedProduct> ResolvedProductConstPtr;
-
-class ResolvedProject;
-typedef std::shared_ptr<ResolvedProject> ResolvedProjectPtr;
-typedef std::shared_ptr<const ResolvedProject> ResolvedProjectConstPtr;
-
-class TopLevelProject;
-typedef std::shared_ptr<TopLevelProject> TopLevelProjectPtr;
-typedef std::shared_ptr<const TopLevelProject> TopLevelProjectConstPtr;
-
-class ResolvedFileContext;
-typedef std::shared_ptr<ResolvedFileContext> ResolvedFileContextPtr;
-typedef std::shared_ptr<const ResolvedFileContext> ResolvedFileContextConstPtr;
-
-class Rule;
-typedef std::shared_ptr<Rule> RulePtr;
-typedef std::shared_ptr<const Rule> RuleConstPtr;
-
-class ResolvedScanner;
-typedef std::shared_ptr<ResolvedScanner> ResolvedScannerPtr;
-typedef std::shared_ptr<const ResolvedScanner> ResolvedScannerConstPtr;
-
-class SourceArtifactInternal;
-typedef std::shared_ptr<SourceArtifactInternal> SourceArtifactPtr;
-typedef std::shared_ptr<const SourceArtifactInternal> SourceArtifactConstPtr;
-
-class ScriptFunction;
-typedef std::shared_ptr<ScriptFunction> ScriptFunctionPtr;
-typedef std::shared_ptr<const ScriptFunction> ScriptFunctionConstPtr;
-
-class RuleArtifact;
-typedef std::shared_ptr<RuleArtifact> RuleArtifactPtr;
-typedef std::shared_ptr<const RuleArtifact> RuleArtifactConstPtr;
-
-class ResolvedModule;
-typedef std::shared_ptr<ResolvedModule> ResolvedModulePtr;
-typedef std::shared_ptr<const ResolvedModule> ResolvedModuleConstPtr;
-
-class ResolvedGroup;
-typedef std::shared_ptr<ResolvedGroup> GroupPtr;
-typedef std::shared_ptr<const ResolvedGroup> GroupConstPtr;
-
-class ArtifactProperties;
-typedef std::shared_ptr<ArtifactProperties> ArtifactPropertiesPtr;
-typedef std::shared_ptr<const ArtifactProperties> ArtifactPropertiesConstPtr;
-
 template <typename T> inline static uint qHash(const std::shared_ptr<T> &p, uint seed = 0)
 {
     return ::qHash(p.get(), seed);
@@ -137,5 +154,6 @@ template <typename T> inline static uint qHash(const std::shared_ptr<T> &p, uint
 
 } // namespace Internal
 } // namespace qbs
+#endif
 
 #endif // QBS_LANG_FORWARD_DECLS_H
