@@ -68,9 +68,10 @@
 #include <tools/stringconstants.h>
 
 #include <QtCore/qdir.h>
-#include <QtCore/qregexp.h>
+#include <QtCore/qregularexpression.h>
 
 #include <algorithm>
+#include <memory>
 #include <queue>
 
 namespace qbs {
@@ -98,7 +99,7 @@ struct ProjectResolver::ProductContext
 {
     ResolvedProductPtr product;
     QString buildDirectory;
-    Item *item;
+    Item *item = nullptr;
     using ArtifactPropertiesInfo = std::pair<ArtifactPropertiesPtr, std::vector<CodeLocation>>;
     QHash<QStringList, ArtifactPropertiesInfo> artifactPropertiesPerFilter;
     ProjectResolver::FileLocations sourceArtifactLocations;
@@ -114,21 +115,19 @@ struct ProjectResolver::ModuleContext
 class CancelException { };
 
 
-ProjectResolver::ProjectResolver(Evaluator *evaluator, const ModuleLoaderResult &loadResult,
-        const SetupProjectParameters &setupParameters, Logger &logger)
+ProjectResolver::ProjectResolver(Evaluator *evaluator, ModuleLoaderResult loadResult,
+        SetupProjectParameters setupParameters, Logger &logger)
     : m_evaluator(evaluator)
     , m_logger(logger)
     , m_engine(m_evaluator->engine())
     , m_progressObserver(nullptr)
-    , m_setupParams(setupParameters)
-    , m_loadResult(loadResult)
+    , m_setupParams(std::move(setupParameters))
+    , m_loadResult(std::move(loadResult))
 {
     QBS_CHECK(FileInfo::isAbsolute(m_setupParams.buildRoot()));
 }
 
-ProjectResolver::~ProjectResolver()
-{
-}
+ProjectResolver::~ProjectResolver() = default;
 
 void ProjectResolver::setProgressObserver(ProgressObserver *observer)
 {
@@ -233,7 +232,7 @@ static void makeSubProjectNamesUniqe(const ResolvedProjectPtr &parentProject)
 TopLevelProjectPtr ProjectResolver::resolveTopLevelProject()
 {
     if (m_progressObserver)
-        m_progressObserver->setMaximum(m_loadResult.productInfos.size());
+        m_progressObserver->setMaximum(int(m_loadResult.productInfos.size()));
     const TopLevelProjectPtr project = TopLevelProject::create();
     project->buildDirectory = TopLevelProject::deriveBuildDirectory(m_setupParams.buildRoot(),
             TopLevelProject::deriveId(m_setupParams.finalBuildConfigurationTree()));
@@ -656,8 +655,8 @@ QVariantMap ProjectResolver::resolveAdditionalModuleProperties(const Item *group
                                                                const QVariantMap &currentValues)
 {
     // Step 1: Retrieve the properties directly set in the group
-    const ModulePropertiesPerGroup &mp = m_loadResult.productInfos.value(m_productContext->item)
-            .modulePropertiesSetInGroups;
+    const ModulePropertiesPerGroup &mp = mapValue(
+            m_loadResult.productInfos, m_productContext->item).modulePropertiesSetInGroups;
     const auto it = mp.find(group);
     if (it == mp.end())
         return {};
@@ -812,7 +811,7 @@ void ProjectResolver::resolveGroupFully(Item *item, ProjectResolver::ProjectCont
         group->targetOfModule = moduleProp->value().toString();
     ErrorInfo fileError;
     if (!patterns.empty()) {
-        group->wildcards = std::unique_ptr<SourceWildCards>(new SourceWildCards);
+        group->wildcards = std::make_unique<SourceWildCards>();
         SourceWildCards *wildcards = group->wildcards.get();
         wildcards->group = group.get();
         wildcards->excludePatterns = m_evaluator->stringListValue(
@@ -826,7 +825,7 @@ void ProjectResolver::resolveGroupFully(Item *item, ProjectResolver::ProjectCont
                                  &m_productContext->sourceArtifactLocations, &fileError);
     }
 
-    for (const QString &fileName : files) {
+    for (const QString &fileName : qAsConst(files)) {
         createSourceArtifact(m_productContext->product, fileName, group, false, filesLocation,
                              &m_productContext->sourceArtifactLocations, &fileError);
     }
@@ -901,7 +900,7 @@ void ProjectResolver::adaptExportedPropertyValues(const Item *shadowProductItem)
     const std::function<QVariant(const QVariantMap &, const QVariant &)> mapper
             = [&stringListMapper, &mapper](
             const QVariantMap &mappings, const QVariant &value) -> QVariant {
-        switch (static_cast<QMetaType::Type>(value.type())) {
+        switch (static_cast<QMetaType::Type>(value.userType())) {
         case QMetaType::QString:
             return stringMapper(mappings, value.toString());
         case QMetaType::QStringList:
@@ -946,7 +945,7 @@ void ProjectResolver::collectExportedProductDependencies()
             }
         }
         const ModuleLoaderResult::ProductInfo &importingProductInfo
-                = m_loadResult.productInfos.value(importingProductItem);
+                = mapValue(m_loadResult.productInfos, importingProductItem);
         const ProductDependencyInfos &depInfos
                 = getProductDependencies(dummyProduct, importingProductInfo);
         for (const auto &dep : depInfos.dependencies) {
@@ -988,7 +987,7 @@ void ProjectResolver::resolveShadowProduct(Item *item, ProjectResolver::ProjectC
     try {
         adaptExportedPropertyValues(item);
     } catch (const ErrorInfo &) {}
-    m_productExportInfo.push_back(std::make_pair(m_productContext->product, item));
+    m_productExportInfo.emplace_back(m_productContext->product, item);
 }
 
 void ProjectResolver::setupExportedProperties(const Item *item, const QString &namePrefix,
@@ -1022,7 +1021,8 @@ void ProjectResolver::setupExportedProperties(const Item *item, const QString &n
         const ItemDeclaration itemDecl
                 = BuiltinDeclarations::instance().declarationsForType(item->type());
         PropertyDeclaration propertyDecl;
-        for (const PropertyDeclaration &decl : itemDecl.properties()) {
+        const auto itemProperties = itemDecl.properties();
+        for (const PropertyDeclaration &decl : itemProperties) {
             if (decl.name() == it.key()) {
                 propertyDecl = decl;
                 exportedProperty.isBuiltin = true;
@@ -1052,12 +1052,12 @@ void ProjectResolver::setupExportedProperties(const Item *item, const QString &n
     std::sort(properties.begin(), properties.end(), less);
 }
 
-static bool usesImport(const ExportedProperty &prop, const QRegExp &regex)
+static bool usesImport(const ExportedProperty &prop, const QRegularExpression &regex)
 {
-    return regex.indexIn(prop.sourceCode) != -1;
+    return prop.sourceCode.indexOf(regex) != -1;
 }
 
-static bool usesImport(const ExportedItem &item, const QRegExp &regex)
+static bool usesImport(const ExportedItem &item, const QRegularExpression &regex)
 {
     return any_of(item.properties,
                   [regex](const ExportedProperty &p) { return usesImport(p, regex); })
@@ -1073,7 +1073,7 @@ static bool usesImport(const ExportedModule &module, const QString &name)
     // (3) var obj = DataCollection;
     const QString pattern = QStringLiteral("\\b%1\\b");
 
-    const QRegExp regex(pattern.arg(name)); // std::regex is much slower
+    const QRegularExpression regex(pattern.arg(name)); // std::regex is much slower
     return any_of(module.m_properties,
                   [regex](const ExportedProperty &p) { return usesImport(p, regex); })
             || any_of(module.children,
@@ -1109,7 +1109,8 @@ void ProjectResolver::resolveExport(Item *exportItem, ProjectContext *)
                                                                  exportItem->file()->content());
         }
     }
-    for (const QString &builtinImport: JsExtensions::extensionNames()) {
+    const auto builtInImports = JsExtensions::extensionNames();
+    for (const QString &builtinImport: builtInImports) {
         if (usesImport(exportedModule, builtinImport))
             exportedModule.importStatements << QStringLiteral("import qbs.") + builtinImport;
     }
@@ -1458,7 +1459,7 @@ void ProjectResolver::matchArtifactProperties(const ResolvedProductPtr &product,
         const std::vector<SourceArtifactPtr> &artifacts)
 {
     for (const SourceArtifactPtr &artifact : artifacts) {
-        for (const ArtifactPropertiesConstPtr &artifactProperties : product->artifactProperties) {
+        for (const auto &artifactProperties : product->artifactProperties) {
             if (!artifact->isTargetOfModule()
                     && artifact->fileTags.intersects(artifactProperties->fileTagsFilter())) {
                 artifact->properties = artifactProperties->propertyMap();
@@ -1609,7 +1610,7 @@ void ProjectResolver::resolveProductDependencies(const ProjectContext &projectCo
             continue;
         Item *productItem = m_productItemMap.value(rproduct);
         const ModuleLoaderResult::ProductInfo &productInfo
-                = m_loadResult.productInfos.value(productItem);
+                = mapValue(m_loadResult.productInfos, productItem);
         const ProductDependencyInfos &depInfos = getProductDependencies(rproduct, productInfo);
         if (depInfos.hasDisabledDependency)
             disabledDependency = true;
@@ -1777,6 +1778,7 @@ void ProjectResolver::evaluateProperty(const Item *item, const QString &propName
         } else if (pd.type() == PropertyDeclaration::VariantList) {
             v = v.toList();
         }
+        checkAllowedValues(v, propValue->location(), pd, propName);
         result[propName] = v;
         break;
     }
@@ -1787,15 +1789,52 @@ void ProjectResolver::evaluateProperty(const Item *item, const QString &propName
         VariantValuePtr vvp = std::static_pointer_cast<VariantValue>(propValue);
         QVariant v = vvp->value();
 
-        if (v.isNull() && !item->propertyDeclaration(propName).isScalar()) // QTBUG-51237
+        const PropertyDeclaration pd = item->propertyDeclaration(propName);
+        if (v.isNull() && !pd.isScalar()) // QTBUG-51237
             v = QStringList();
 
+        checkAllowedValues(v, propValue->location(), pd, propName);
         result[propName] = v;
         break;
     }
     }
 }
 
+void ProjectResolver::checkAllowedValues(
+        const QVariant &value, const CodeLocation &loc, const PropertyDeclaration &decl,
+        const QString &key) const
+{
+    const auto type = decl.type();
+    if (type != PropertyDeclaration::String && type != PropertyDeclaration::StringList)
+        return;
+
+    if (value.isNull())
+        return;
+
+    const auto &allowedValues = decl.allowedValues();
+    if (allowedValues.isEmpty())
+        return;
+
+    const auto checkValue = [this, &loc, &allowedValues, &key](const QString &value)
+    {
+        if (!allowedValues.contains(value)) {
+            const auto message = Tr::tr("Value '%1' is not allowed for property '%2'.")
+                    .arg(value, key);
+            ErrorInfo error(message, loc);
+            handlePropertyError(error, m_setupParams, m_logger);
+        }
+    };
+
+    if (type == PropertyDeclaration::StringList) {
+        const auto strings = value.toStringList();
+        for (const auto &string: strings) {
+            checkValue(string);
+        }
+    } else if (type == PropertyDeclaration::String) {
+        checkValue(value.toString());
+    }
+}
+
 void ProjectResolver::collectPropertiesForExportItem(const QualifiedId &moduleName,
          const ValuePtr &value, Item *moduleInstance, QVariantMap &moduleProps)
 {
