@@ -48,8 +48,9 @@
 #include <QtCore/qdatastream.h>
 #include <QtCore/qflags.h>
 #include <QtCore/qprocess.h>
-#include <QtCore/qregexp.h>
+#include <QtCore/qregularexpression.h>
 #include <QtCore/qstring.h>
+#include <QtCore/qstringlist.h>
 #include <QtCore/qvariant.h>
 
 #include <memory>
@@ -128,7 +129,6 @@ public:
     void load(const QString &filePath);
     void setupWriteStream(const QString &filePath);
     void finalizeWriteStream();
-    void closeStream();
     void clear();
 
     const HeadData &headData() const { return m_headData; }
@@ -167,22 +167,23 @@ private:
     static const PersistentObjectId ValueNotFoundId = -1;
     static const PersistentObjectId EmptyValueId = -2;
 
+    std::unique_ptr<QIODevice> m_file;
     QDataStream m_stream;
     HeadData m_headData;
     std::vector<void *> m_loadedRaw;
     std::vector<std::shared_ptr<void>> m_loaded;
-    QHash<const void*, int> m_storageIndices;
-    PersistentObjectId m_lastStoredObjectId;
+    std::unordered_map<const void*, int> m_storageIndices;
+    PersistentObjectId m_lastStoredObjectId = 0;
 
     std::vector<QString> m_stringStorage;
     QHash<QString, int> m_inverseStringStorage;
-    PersistentObjectId m_lastStoredStringId;
+    PersistentObjectId m_lastStoredStringId = 0;
     std::vector<QProcessEnvironment> m_envStorage;
     QHash<QProcessEnvironment, int> m_inverseEnvStorage;
-    PersistentObjectId m_lastStoredEnvId;
+    PersistentObjectId m_lastStoredEnvId = 0;
     std::vector<QStringList> m_stringListStorage;
     QHash<QStringList, int> m_inverseStringListStorage;
-    PersistentObjectId m_lastStoredStringListId;
+    PersistentObjectId m_lastStoredStringListId = 0;
     Logger &m_logger;
 
     template<typename T, typename Enable>
@@ -198,14 +199,14 @@ template<typename T> inline void PersistentPool::storeSharedObject(const T *obje
         return;
     }
     const void * const addr = uniqueAddress(object);
-    PersistentObjectId id = m_storageIndices.value(addr, -1);
-    if (id < 0) {
-        id = m_lastStoredObjectId++;
-        m_storageIndices.insert(addr, id);
+    const auto found = m_storageIndices.find(addr);
+    if (found == m_storageIndices.end()) {
+        PersistentObjectId id = m_lastStoredObjectId++;
+        m_storageIndices[addr] = id;
         m_stream << id;
         store(*object);
     } else {
-        m_stream << id;
+        m_stream << found->second;
     }
 }
 
@@ -340,8 +341,8 @@ struct PPHelper
 /***** Specializations of Helper class *****/
 
 template<typename T>
-struct PPHelper<T, std::enable_if_t<std::is_member_function_pointer<
-        decltype(&T::template completeSerializationOp<PersistentPool::Load>)>::value>>
+struct PPHelper<T, std::enable_if_t<std::is_member_function_pointer_v<
+        decltype(&T::template completeSerializationOp<PersistentPool::Load>)>>>
 {
     static void store(const T &value, PersistentPool *pool)
     {
@@ -353,7 +354,7 @@ struct PPHelper<T, std::enable_if_t<std::is_member_function_pointer<
     }
 };
 
-template<typename T> struct PPHelper<T, std::enable_if_t<std::is_integral<T>::value>>
+template<typename T> struct PPHelper<T, std::enable_if_t<std::is_integral_v<T>>>
 {
     static void store(const T &value, PersistentPool *pool) { pool->m_stream << value; }
     static void load(T &value, PersistentPool *pool) { pool->m_stream >> value; }
@@ -370,7 +371,7 @@ template<> struct PPHelper<long>
     }
 };
 
-template<typename T> struct PPHelper<T, std::enable_if_t<std::is_enum<T>::value>>
+template<typename T> struct PPHelper<T, std::enable_if_t<std::is_enum_v<T>>>
 {
     using U = std::underlying_type_t<T>;
     static void store(const T &value, PersistentPool *pool)
@@ -413,8 +414,8 @@ template<typename T> struct PPHelper<T *>
     static void load(T* &value, PersistentPool *pool) { value = pool->idLoad<T>(); }
 };
 
-template<typename T> struct PPHelper<T, std::enable_if_t<std::is_same<T, QString>::value
-        || std::is_same<T, QStringList>::value || std::is_same<T, QProcessEnvironment>::value>>
+template<typename T> struct PPHelper<T, std::enable_if_t<std::is_same_v<T, QString>
+        || std::is_same_v<T, QStringList> || std::is_same_v<T, QProcessEnvironment>>>
 {
     static void store(const T &v, PersistentPool *pool) { pool->idStoreValue(v); }
     static void load(T &v, PersistentPool *pool) { v = pool->idLoadValue<T>(); }
@@ -426,10 +427,16 @@ template<> struct PPHelper<QVariant>
     static void load(QVariant &v, PersistentPool *pool) { v = pool->loadVariant(); }
 };
 
-template<> struct PPHelper<QRegExp>
+template<> struct PPHelper<QRegularExpression>
 {
-    static void store(const QRegExp &re, PersistentPool *pool) { pool->store(re.pattern()); }
-    static void load(QRegExp &re, PersistentPool *pool) { re.setPattern(pool->load<QString>()); }
+    static void store(const QRegularExpression &re, PersistentPool *pool)
+    {
+        pool->store(re.pattern());
+    }
+    static void load(QRegularExpression &re, PersistentPool *pool)
+    {
+        re.setPattern(pool->load<QString>());
+    }
 };
 
 template<typename T, typename U> struct PPHelper<std::pair<T, U>>
@@ -461,6 +468,7 @@ template<typename T> struct PPHelper<QFlags<T>>
 
 template<typename T> struct IsSimpleContainer : std::false_type { };
 template<typename T> struct IsSimpleContainer<QList<T>> : std::true_type { };
+template<> struct IsSimpleContainer<QStringList> : std::false_type { };
 template<typename T> struct IsSimpleContainer<std::vector<T>> : std::true_type { };
 
 template<typename T> struct PPHelper<T, std::enable_if_t<IsSimpleContainer<T>::value>>
@@ -490,7 +498,7 @@ struct PPHelper<T, std::enable_if_t<IsKeyValueContainer<T>::value>>
 {
     static void store(const T &container, PersistentPool *pool)
     {
-        pool->store(container.size());
+        pool->store(int(container.size()));
         for (auto it = container.cbegin(); it != container.cend(); ++it) {
             pool->store(it.key());
             pool->store(it.value());
