@@ -1,6 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2019 Jochen Ulrich <jochenulrich@t-online.de>
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
@@ -39,6 +40,8 @@ public:
     TestBlackbox();
 
 private slots:
+    void allowedValues();
+    void allowedValues_data();
     void addFileTagToGeneratedArtifact();
     void alwaysRun();
     void alwaysRun_data();
@@ -48,6 +51,8 @@ private slots:
     void artifactScanning();
     void assembly();
     void autotestWithDependencies();
+    void autotestTimeout();
+    void autotestTimeout_data();
     void autotests_data();
     void autotests();
     void auxiliaryInputsFromDependencies();
@@ -57,6 +62,10 @@ private slots:
     void buildDirectories();
     void buildEnvChange();
     void buildGraphVersions();
+    void buildVariantDefaults_data();
+    void buildVariantDefaults();
+    void capnproto();
+    void capnproto_data();
     void changedFiles_data();
     void changedFiles();
     void changedInputsFromDependencies();
@@ -80,9 +89,10 @@ private slots:
     void conflictingArtifacts();
     void cxxLanguageVersion();
     void cxxLanguageVersion_data();
+    void conanfileProbe();
     void cpuFeatures();
     void dependenciesProperty();
-    void dependencyProfileMismatch();
+    void dependencyScanningLoop();
     void deprecatedProperty();
     void disappearedProfile();
     void discardUnusedData();
@@ -93,6 +103,7 @@ private slots:
     void dynamicMultiplexRule();
     void dynamicProject();
     void dynamicRuleOutputs();
+    void emptyProfile();
     void enableExceptions();
     void enableExceptions_data();
     void enableRtti();
@@ -115,10 +126,15 @@ private slots:
     void externalLibs();
     void fileDependencies();
     void fileTagsFilterMerging();
+    void freedesktop();
     void generatedArtifactAsInputToDynamicRule();
+    void generateLinkerMapFile();
     void generator();
     void generator_data();
     void groupsInModules();
+    void grpc_data();
+    void grpc();
+    void hostOsProperties();
     void ico();
     void importAssignment();
     void importChangeTracking();
@@ -158,6 +174,7 @@ private slots:
     void jsExtensionsTemporaryDir();
     void jsExtensionsTextFile();
     void jsExtensionsBinaryFile();
+    void lastModuleCandidateBroken();
     void ld();
     void linkerMode();
     void linkerVariant_data();
@@ -165,7 +182,10 @@ private slots:
     void lexyacc();
     void lexyaccOutputs();
     void lexyaccOutputs_data();
+    void linkerLibraryDuplicates();
+    void linkerLibraryDuplicates_data();
     void linkerScripts();
+    void linkerModuleDefinition();
     void listProducts();
     void listPropertiesWithOuter();
     void listPropertyOrder();
@@ -204,7 +224,10 @@ private slots:
     void nsisDependencies();
     void outOfDateMarking();
     void outputArtifactAutoTagging();
+    void outputRedirection();
     void overrideProjectProperties();
+    void pathProbe_data();
+    void pathProbe();
     void pchChangeTracking();
     void perGroupDefineInExportItem();
     void pkgConfigProbe();
@@ -212,6 +235,7 @@ private slots:
     void pkgConfigProbeSysroot();
     void pluginDependency();
     void precompiledAndPrefixHeaders();
+    void precompiledHeaderAndRedefine();
     void preventFloatingPointValues();
     void probeChangeTracking();
     void probeProperties();
@@ -220,17 +244,21 @@ private slots:
     void probesAndArrayProperties();
     void probesInNestedModules();
     void productDependenciesByType();
+    void productInExportedModule();
     void productProperties();
     void propertyAssignmentOnNonPresentModule();
     void propertyAssignmentInFailedModule();
     void propertyChanges();
+    void propertyEvaluationContext();
     void propertyPrecedence();
     void properQuoting();
     void propertiesInExportItems();
     void protobuf_data();
     void protobuf();
+    void protobufLibraryInstall();
     void pseudoMultiplexing();
     void qbsConfig();
+    void qbsSession();
     void qbsVersion();
     void qtBug51237();
     void radAfterIncompleteBuild();
@@ -238,6 +266,8 @@ private slots:
     void recursiveRenaming();
     void recursiveWildcards();
     void referenceErrorInExport();
+    void removeDuplicateLibraries_data();
+    void removeDuplicateLibraries();
     void reproducibleBuild();
     void reproducibleBuild_data();
     void require();
@@ -250,7 +280,11 @@ private slots:
     void ruleCycle();
     void ruleWithNoInputs();
     void ruleWithNonRequiredInputs();
+    void sanitizer_data();
+    void sanitizer();
     void scannerItem();
+    void scanResultInOtherProduct();
+    void scanResultInNonDependency();
     void setupBuildEnvironment();
     void setupRunEnvironment();
     void smartRelinking();
@@ -270,6 +304,7 @@ private slots:
     void suspiciousCalls();
     void suspiciousCalls_data();
     void systemIncludePaths();
+    void distributionIncludePaths();
     void systemRunPaths();
     void systemRunPaths_data();
     void tar();
@@ -284,8 +319,10 @@ private slots:
     void trackRemoveFile();
     void trackRemoveFileTag();
     void trackRemoveProduct();
+    void transitiveInvalidDependencies();
     void transitiveOptionalDependencies();
     void typescript();
+    void undefinedTargetPlatform();
     void usingsAsSoleInputsNonMultiplexed();
     void variantSuffix();
     void variantSuffix_data();
