@@ -79,7 +79,7 @@ void TestBlackboxAndroid::android()
     QFETCH(QString, projectDir);
     QFETCH(QStringList, productNames);
     QFETCH(QList<QByteArrayList>, expectedFilesLists);
-    QFETCH(QStringList, qmlAppCustomProperties);
+    QFETCH(QStringList, customProperties);
     QFETCH(bool, enableAapt2);
     QFETCH(bool, generateAab);
     QFETCH(bool, isIncrementalBuild);
@@ -112,6 +112,8 @@ void TestBlackboxAndroid::android()
 
     const QString buildSubDir = enableAapt2 ? (generateAab ? "aab" : "aapt2") : "aapt";
     QDir::setCurrent(testDataDir + "/" + projectDir);
+    if (!isIncrementalBuild)
+        rmDirR(relativeBuildDir(buildSubDir));
 
     static const QStringList configNames { "debug", "release" };
     for (const QString &configName : configNames) {
@@ -119,7 +121,7 @@ void TestBlackboxAndroid::android()
         const QString configArgument = "config:" + configName;
         QbsRunParameters resolveParams("resolve");
         resolveParams.buildDirectory = buildSubDir;
-        resolveParams.arguments << configArgument << qmlAppCustomProperties;
+        resolveParams.arguments << configArgument << customProperties;
         resolveParams.profile = p.name();
         QCOMPARE(runQbs(resolveParams), 0);
         QbsRunParameters buildParams(QStringList{"--command-echo-mode", "command-line",
@@ -266,20 +268,25 @@ void TestBlackboxAndroid::android_data()
         return result;
     };
 
-    auto commonFiles = [](bool generateAab) {
+    auto commonFiles = [](bool generateAab, bool codeSign = true,
+            QString keyAlias="androiddebugkey") {
+        QByteArrayList files;
         if (generateAab)
-            return (QByteArrayList()
-                    << "base/manifest/AndroidManifest.xml" << "base/dex/classes.dex"
-                    << "BundleConfig.pb");
-        return (QByteArrayList()
-                << "AndroidManifest.xml" << "META-INF/ANDROIDD.RSA" << "META-INF/ANDROIDD.SF"
-                << "META-INF/MANIFEST.MF" << "classes.dex");
+            files << "base/manifest/AndroidManifest.xml" << "base/dex/classes.dex"
+                  << "BundleConfig.pb";
+        else
+            files << "AndroidManifest.xml" << "classes.dex";
+        if (codeSign)
+            files << QByteArray("META-INF/" + keyAlias.toUpper().left(8).toUtf8() + ".RSA")
+                  << QByteArray("META-INF/" + keyAlias.toUpper().left(8).toUtf8() + ".SF")
+                  << "META-INF/MANIFEST.MF";
+        return files;
     };
 
     QTest::addColumn<QString>("projectDir");
     QTest::addColumn<QStringList>("productNames");
     QTest::addColumn<QList<QByteArrayList>>("expectedFilesLists");
-    QTest::addColumn<QStringList>("qmlAppCustomProperties");
+    QTest::addColumn<QStringList>("customProperties");
     QTest::addColumn<bool>("enableAapt2");
     QTest::addColumn<bool>("generateAab");
     QTest::addColumn<bool>("isIncrementalBuild");
@@ -294,10 +301,11 @@ void TestBlackboxAndroid::android_data()
     bool generateAab = false;
     bool isIncrementalBuild = false;
 
-    auto qtAppExpectedFiles = [&](bool generateAab, bool enableAapt2) {
+    auto qtAppExpectedFiles = [&](bool generateAab, bool enableAapt2, bool codeSign = true,
+            QString keyAlias="androiddebugkey") {
         QByteArrayList expectedFile;
         if (singleArchQt) {
-            expectedFile << commonFiles(generateAab) + expandArchs(ndkArchsForQt, {
+            expectedFile << commonFiles(generateAab, codeSign, keyAlias) + expandArchs(ndkArchsForQt, {
                        cxxLibPath("libgnustl_shared.so", true),
                        "assets/--Added-by-androiddeployqt--/qt_cache_pregenerated_file_list",
                        "lib/${ARCH}/libplugins_imageformats_libqgif.so",
@@ -315,7 +323,7 @@ void TestBlackboxAndroid::android_data()
                        "lib/${ARCH}/libQt5Widgets.so",
                        "lib/${ARCH}/libqt-app.so"}, generateAab);
         } else {
-            expectedFile << commonFiles(generateAab) + expandArchs(ndkArchsForQt, {
+            expectedFile << commonFiles(generateAab, codeSign, keyAlias) + expandArchs(ndkArchsForQt, {
                        cxxLibPath("libgnustl_shared.so", true),
                        "lib/${ARCH}/libplugins_imageformats_qgif_${ARCH}.so",
                        "lib/${ARCH}/libplugins_imageformats_qico_${ARCH}.so",
@@ -327,7 +335,7 @@ void TestBlackboxAndroid::android_data()
                        "lib/${ARCH}/libqt-app_${ARCH}.so"}, generateAab);
         }
         if (generateAab)
-            expectedFile << "base/resources.pb" << "base/assets.pb" << "base/native.pb";
+            expectedFile << "base/resources.pb" << "base/native.pb";
         else
             expectedFile << "resources.arsc";
         if (version >= qbs::Version(5, 14))
@@ -344,12 +352,94 @@ void TestBlackboxAndroid::android_data()
             expectedFile << "res/layout/splash.xml";
         return expectedFile;
     };
+    auto codeSignProperties = [&](bool codeSign, QString keyStorePath, QString keystorePassword,
+            QString keyPassword, QString keyAlias) {
+        if (!codeSign)
+            return QStringList{"modules.codesign.enableCodeSigning:false"};
+        return QStringList{
+            "modules.codesign.enableCodeSigning:true",
+            "modules.codesign.keystorePath:" + keyStorePath,
+            "modules.codesign.keystorePassword:" + keystorePassword,
+            "modules.codesign.keyPassword:" + keyPassword,
+            "modules.codesign.keyAlias:" + keyAlias,
+        };
+    };
+    bool codeSign = true;
+    QString keyStorePath(testDataDir + "/qt-app/test.keystore");
+    QString keystorePassword("qbsKeystoreTest");
+    QString keyPassword("qbsKeyTest");
+    QString keyAlias("qbsTest");
     QTest::newRow("qt app")
             << "qt-app" << QStringList("qt-app")
             << (QList<QByteArrayList>() << (QByteArrayList() << qtAppExpectedFiles(generateAab,
-                                                                                   enableAapt2)))
-            << QStringList{aaptVersion(enableAapt2), packageType(generateAab)}
+                                                                                   enableAapt2,
+                                                                                   codeSign,
+                                                                                   keyAlias)))
+            << (QStringList() << codeSignProperties(codeSign, keyStorePath, keystorePassword,
+                                                    keyPassword, keyAlias)
+                << aaptVersion(enableAapt2)
+                << packageType(generateAab))
             << enableAapt2 << generateAab << isIncrementalBuild;
+    codeSign = false;
+    QTest::newRow("qt app no signing")
+            << "qt-app" << QStringList("qt-app")
+            << (QList<QByteArrayList>() << (QByteArrayList() << qtAppExpectedFiles(generateAab,
+                                                                                   enableAapt2,
+                                                                                   codeSign,
+                                                                                   keyAlias)))
+            << (QStringList() << codeSignProperties(codeSign, keyStorePath, keystorePassword,
+                                                    keyPassword, keyAlias)
+                << aaptVersion(enableAapt2)
+                << packageType(generateAab))
+            << enableAapt2 << generateAab << isIncrementalBuild;
+    enableAapt2 = true;
+    codeSign = true;
+    QTest::newRow("qt app aapt2")
+            << "qt-app" << QStringList("qt-app")
+            << (QList<QByteArrayList>() << (QByteArrayList() << qtAppExpectedFiles(generateAab,
+                                                                                   enableAapt2,
+                                                                                   codeSign,
+                                                                                   keyAlias)))
+            << (QStringList() << codeSignProperties(codeSign, keyStorePath, keystorePassword,
+                                                    keyPassword, keyAlias)
+                << aaptVersion(enableAapt2)
+                << packageType(generateAab))
+            << enableAapt2 << generateAab << isIncrementalBuild;
+    generateAab = true;
+    QTest::newRow("qt app aab")
+            << "qt-app" << QStringList("qt-app")
+            << (QList<QByteArrayList>() << (QByteArrayList() << qtAppExpectedFiles(generateAab,
+                                                                                   enableAapt2,
+                                                                                   codeSign,
+                                                                                   keyAlias)))
+            << (QStringList() << codeSignProperties(codeSign, keyStorePath, keystorePassword,
+                                                    keyPassword, keyAlias)
+                << aaptVersion(enableAapt2)
+                << packageType(generateAab))
+            << enableAapt2 << generateAab << isIncrementalBuild;
+    codeSign = false;
+    QTest::newRow("qt app aab no signing")
+            << "qt-app" << QStringList("qt-app")
+            << (QList<QByteArrayList>() << (QByteArrayList() << qtAppExpectedFiles(generateAab,
+                                                                                   enableAapt2,
+                                                                                   codeSign,
+                                                                                   keyAlias)))
+            << (QStringList() << codeSignProperties(codeSign, keyStorePath, keystorePassword,
+                                                    keyPassword, keyAlias)
+                << aaptVersion(enableAapt2)
+                << packageType(generateAab))
+            << enableAapt2 << generateAab << isIncrementalBuild;
+
+    const QByteArrayList ndkArchsForQtSave = ndkArchsForQt;
+    ndkArchsForQt = {ndkArchsForQt.first()};
+    QTest::newRow("qt app (single arch)")
+            << "qt-app" << QStringList("qt-app")
+            << (QList<QByteArrayList>() << (QByteArrayList() << qtAppExpectedFiles(generateAab,
+                                                                                   enableAapt2)))
+            << QStringList{aaptVersion(enableAapt2), packageType(generateAab),
+                           "modules.qbs.architectures:" + archsForQt.first()}
+            << enableAapt2 << generateAab << isIncrementalBuild;
+    ndkArchsForQt = ndkArchsForQtSave;
 
     auto teaPotAppExpectedFiles = [&](const QByteArrayList &archs, bool generateAab) {
         QByteArrayList expectedFile;
@@ -370,6 +460,8 @@ void TestBlackboxAndroid::android_data()
         return expectedFile;
     };
 
+    generateAab = false;
+    enableAapt2 = false;
     QTest::newRow("teapot")
             << "teapot" << QStringList("TeapotNativeActivity")
             << (QList<QByteArrayList>() << teaPotAppExpectedFiles(archs, generateAab))
@@ -424,6 +516,7 @@ void TestBlackboxAndroid::android_data()
                            "modules.qbs.architecture:" + archsStringList.first(),
                            aaptVersion(enableAapt2), packageType(generateAab)}
             << enableAapt2 << generateAab << isIncrementalBuild;
+
     auto qmlAppExpectedFiles = [&](bool generateAab, bool enableAapt2) {
         QByteArrayList expectedFile;
         if (singleArchQt) {
@@ -717,7 +810,6 @@ void TestBlackboxAndroid::android_data()
     } else {
         qmlAppCustomProperties = QStringList{"modules.Android.sdk.automaticSources:false"};
     }
-
     // aapt tool for the resources works with a directory option pointing to the parent directory
     // of the resources (res).
     // The Qt.android_support module adds res/values/libs.xml (from Qt install dir). So the res from
@@ -838,6 +930,7 @@ void TestBlackboxAndroid::android_data()
             expectedFile << "resources.arsc";
         return expectedFile;
     };
+
     QTest::newRow("no native")
             << "no-native"
             << QStringList("com.example.android.basicmediadecoder")
